export class Velocity {
    constructor(horizontal, vertical) {
        this.horizontal = horizontal;
        this.vertical = vertical;
    }
    get angle() {
        return Math.atan2(this.vertical, this.horizontal);
    }
    set angle(angle) {
        const length = this.length;
        this.horizontal = Math.cos(angle) * length;
        this.vertical = Math.sin(angle) * length;
    }
    get length() {
        return Math.sqrt(Math.pow(this.horizontal, 2) + Math.pow(this.vertical, 2));
    }
    set length(length) {
        const angle = this.angle;
        this.horizontal = Math.cos(angle) * length;
        this.vertical = Math.sin(angle) * length;
    }
    add(v) {
        return new Velocity(this.horizontal + v.horizontal, this.vertical + v.vertical);
    }
    addTo(v) {
        this.horizontal += v.horizontal;
        this.vertical += v.vertical;
    }
    sub(v) {
        return new Velocity(this.horizontal - v.horizontal, this.vertical - v.vertical);
    }
    subFrom(v) {
        this.horizontal -= v.horizontal;
        this.vertical -= v.vertical;
    }
    mult(n) {
        return new Velocity(this.horizontal * n, this.vertical * n);
    }
    multTo(n) {
        this.horizontal *= n;
        this.vertical *= n;
    }
    div(n) {
        return new Velocity(this.horizontal / n, this.vertical / n);
    }
    divTo(n) {
        this.horizontal /= n;
        this.vertical /= n;
    }
    distanceTo(v) {
        return this.sub(v).length;
    }
    getLengthSq() {
        return Math.pow(this.horizontal, 2) + Math.pow(this.vertical, 2);
    }
    distanceToSq(v) {
        return this.sub(v).getLengthSq();
    }
    manhattanDistanceTo(v) {
        return Math.abs(v.horizontal - this.horizontal) + Math.abs(v.vertical - this.vertical);
    }
    copy() {
        return new Velocity(this.horizontal, this.vertical);
    }
    setTo(velocity) {
        this.horizontal = velocity.horizontal;
        this.vertical = velocity.vertical;
    }
    rotate(angle) {
        return new Velocity(this.horizontal * Math.cos(angle) - this.vertical * Math.sin(angle), this.horizontal * Math.sin(angle) + this.vertical * Math.cos(angle));
    }
}
